/*
* amx_reservation = 1 or 2 or 3
*   Set your server max_players to 1 above the desired value (21 instead of 20 for instance).
*   Query programs will see max_players-1 as max. Admins with reservation
*   can connect when there is 20/20 and player with worst ping or shortest playing time
*   or longest playing time will be kicked/redirected.
*
* Cvars:
*
* amx_reservation <value>
* 0 - Disables the plugin.
* 1 - Kicks the Player with the shortest playing time when an admin connects to a full server.
* 2 - Kicks the Player with the longest playing time when an admin connects to a full server.
* 3 - Kicks the Player with the highest ping when an admin connects to a full server.
* 4 - Normal AMX Mode: 'amx_reservedslots' are reserved, nobody is kicked when an admin connects to a full server
*
* amx_reservedslots <value> : number of reserved slots (if amx_reservation is set to 4)
*
* amx_hideslots <1 or 0> : hide reserved slots or not (if amx_reservation is set to 4)
*
* amx_rslot_redirect <1 or 0> : if 1 will redirect player, if 0 - kick
*
* amx_rslot_redirectip <server.ip.address.and:port> : set server ip/domain and port to where player will be redirected (if amx_rslot_redirect is set to 1)
*
* amx_rslot_redirectpw <password> : set password of server to where player will be redirected (if amx_rslot_redirect is set to 1)
*
* amx_rslot_sounds <1 or 0> : if 1 server will play "ambience/wren1" on all clients when player gets dropped from admin connection (if amx_reservation from 1 to 3)
*
* amx_rslot_hltvimm <1 or 0> : if 1 hltvs will have the same efect as admins with reservation access flag
*
* amx_rslot_hltv_banin <value> : ban hltv if it dosn't disconnect in X seconds, min 10, max 120, (if amx_rslot_hltvimm is set to 0)
* 
* amx_rslot_hltv_bantime <value> : ban time in minutes, min 2, max 60, (if amx_rslot_hltvimm is set to 0)
*
*/

/* History log:
*
* v1.0
*  ~ First version
*  - Changed: from amx to amxx
*  - Added: No anti-reservation scrpts are working with it
*  - Added: Kickes bots/hltv to
*  - Added: Redirection
*
* v1.1
*  - Fixed: Redirection
*
* v1.2
*  - Fixed: Bug with not kicking bots if redirection is enabled
*
* v1.3
*  - Fixed: Lang is working now
*
* v1.4
*  - Removed: Loging of player that was kicked (when amx_reservation from 1 to 3)
*  - Readded: Lang is printing to admin and all players of who was kicked when
*             admin connects to a full server (if amx_reservation from 1 to 3)
*
* v1.5
*  - Added: Sound when player gets dropped if admin is connecting to a full server
*           (if amx_reservation from 1 to 3)
*
* v1.6
*  - Changed: Sound dosn't play to users that are connecting, to avoid sound bug
*
* v1.7
*  - Added: Cvars: amx_rslot_hltvimm (1 - hltvs have immunity), amx_rslot_hltv_banin
*           (ban hltv if it dosn't disconnect in X seconds, min 10, max 120, if
*           amx_rslot_hltvimm 0), amx_rslot_hltv_bantime (ban time in minutes,
*           min 2, max 60, if amx_rslot_hltvimm 0) P.S. I have no mercy
*
*/

#define PLUGIN "Admin Slots Ultimate"
#define VERSION "1.7"
#define AUTHOR "NumB & Slmclarengt"

#include <amxmodx>
#include <amxmisc>
new g_maxplayers
new cvar_redirectip
new cvar_redirectpw
new amx_rslot_hltv_banin
new amx_rslot_hltv_bantime
new bool:g_ban[33]

public plugin_init() {
	register_plugin(PLUGIN, VERSION, AUTHOR)
	register_cvar("amx_reservation","0")
	register_cvar("amx_reservedslots","0")
	register_cvar("amx_hideslots","0")
	register_cvar("amx_rslot_redirect","0")
	register_cvar("amx_rslot_sounds","1")
	register_cvar("amx_rslot_hltvimm","1")
	amx_rslot_hltv_bantime = register_cvar("amx_rslot_hltv_bantime", "3")
	amx_rslot_hltv_banin = register_cvar("amx_rslot_hltv_banin", "30")
	cvar_redirectip = register_cvar("amx_rslot_redirectip"," ")
	cvar_redirectpw = register_cvar("amx_rslot_redirectpw"," ")
	register_dictionary("adminslots_ultimate.txt")
	
	g_maxplayers = get_maxplayers()
}

public plugin_cfg() {
	new amx_reservation_var = get_cvar_num("amx_reservation")
	new amx_hideslots = get_cvar_num("amx_hideslots")
	new amx_reservedslots = get_cvar_num("amx_reservedslots")
	if(amx_reservation_var > 0 && amx_reservation_var < 4) {
		set_cvar_num("sv_visiblemaxplayers", g_maxplayers - 1)
	} else if(amx_reservation_var == 4) {
		if(amx_hideslots == 1) {
			new players = get_playersnum(1)
			new limit = g_maxplayers - amx_reservedslots
			setVisibleSlots(players, limit)
		}
	}
}

kick(id) {
	if (is_user_hltv(id)) {
		hltvwarn(id)
	} else {
		new amx_rslot_redirect = get_cvar_num("amx_rslot_redirect")
		if(amx_rslot_redirect == 1) {
			redirect(id)
		} else {
			new lReason[128]
			format(lReason, 127, "%L", id, "DROPPED_RESERV")
			server_cmd("kick #%d %s", get_user_userid(id), lReason)
		}
	}
}

kickS(id) {
	if (is_user_hltv(id)) {
		hltvwarn(id)
	} else {
		if (get_cvar_num("amx_rslot_sounds") == 1) {
			new players[32], num 
			get_players(players, num) 
			for (new i; i < num; ++i) { 
				client_cmd(i, "spk ^"ambience/wren1^"")
			}
		}
		new amx_rslot_redirect = get_cvar_num("amx_rslot_redirect")
		if(amx_rslot_redirect == 1) {
			redirect(id)
		} else {
			new lReason[128]
			format(lReason, 127, "%L", id, "DROPPED_SHORTEST")
			server_cmd("kick #%d %s", get_user_userid(id), lReason)
		}
	}
}
kickL(id) {
	if (is_user_hltv(id)) {
		hltvwarn(id)
	} else {
		if (get_cvar_num("amx_rslot_sounds") == 1) {
			new players[32], num 
			get_players(players, num) 
			for (new i; i < num; ++i) { 
				client_cmd(i, "spk ^"ambience/wren1^"")
			}
		}
		new amx_rslot_redirect = get_cvar_num("amx_rslot_redirect")
		if(amx_rslot_redirect == 1) {
			redirect(id)
		} else {
			new lReason[128]
			format(lReason, 127, "%L", id, "DROPPED_LONGEST")
			server_cmd("kick #%d %s", get_user_userid(id), lReason)
		}
	}
}
kickP(id) {
	if (is_user_hltv(id)) {
		hltvwarn(id)
	} else {
		if (get_cvar_num("amx_rslot_sounds") == 1) {
			new players[32], num 
			get_players(players, num) 
			for (new i; i < num; ++i) { 
				client_cmd(i, "spk ^"ambience/wren1^"")
			}
		}
		new amx_rslot_redirect = get_cvar_num("amx_rslot_redirect")
		if(amx_rslot_redirect == 1) {
			redirect(id)
		} else {
			new lReason[128]
			format(lReason, 127, "%L", id, "DROPPED_HIGHPING")
			server_cmd("kick #%d %s", get_user_userid(id), lReason)
		}
	}
}

redirect(id) {
	if (is_user_bot(id)) {
		new lReason[128]
		format(lReason, 127, "%L", id, "DROPPED_RESERV")
		server_cmd("kick #%d %s", get_user_userid(id), lReason)
	} else {
		new redirectip[64], redirectpw[32]
		get_pcvar_string(cvar_redirectip,redirectip,63)
		get_pcvar_string(cvar_redirectpw,redirectpw,31)
		client_cmd(id,"echo;echo -=====-;echo Redirecting...;echo -=====-;echo;wait;disconnect;password ^"%s^";connect %s",redirectpw,redirectip)
	}
}

hltvwarn(id) {
	if (get_pcvar_num(amx_rslot_hltv_banin) < 10) {
		set_task(10.0,"banHLTV", id)
		server_cmd("amx_rslot_hltv_banin 10")
	} else if (get_pcvar_num(amx_rslot_hltv_banin) > 120) {
		set_task(120.0,"banHLTV", id)
		server_cmd("amx_rslot_hltv_banin 120")
	} else {
		set_task(get_pcvar_num(amx_rslot_hltv_banin),"banHLTV", id)
	}
	g_ban[id] = true
	new hltvip[32]  
	get_user_ip(id, hltvip, 31, 1)
	new launcherid = 0
	for(new i = 1; i <= g_maxplayers; ++i) {
		if(!is_user_connected(i) && !is_user_connecting(i))
			continue
		if(is_user_hltv(i))
			continue
		new launcherip[32]   
		get_user_ip(i, launcherip, 31, 1)
		if(equali(hltvip, launcherip)) {
		launcherid = i
		}
	}
	if(launcherid) {
		if(get_user_flags(launcherid) & ADMIN_RESERVATION) {
			g_ban[id] = false
			server_cmd("amx_rslot_hltvimm 1")
			set_task(0.5,"launcherIsAdmin")
			set_task(1.0,"disableHLTVImm")
		} else {
			set_task(0.5,"launcherDetected", launcherid)
		}
	} else {
		set_task(1.0,"launcheridNotFound", id)
	}
	return launcherid
}

public launcherIsAdmin() {
	switch(get_cvar_num("amx_reservation")) {
		case 1:
		kickShortestPlayingTime()
		case 2:
		kickLongestPlayingTime()
		case 3:
		kickLag()
		case 4:
		setVisibleSlotsP()
	}
}

public disableHLTVImm() {
	set_cvar_num("amx_rslot_hltvimm", 0)
}

public launcherDetected(id) {
	new banin[64], bantime[32]
	get_pcvar_string(amx_rslot_hltv_banin,banin,63)
	get_pcvar_string(amx_rslot_hltv_bantime,bantime,31)
	set_hudmessage(200, 0, 0, -1.0, 0.30, 0, 6.0, 6.0, 0.5, 0.15, -1)
	show_hudmessage(id, "%L", id, "WARN_LAUNCHER", banin, bantime)
}

public launcheridNotFound(id) { // this id is of HLTV
	new banin[64], bantime[32]
	get_pcvar_string(amx_rslot_hltv_banin,banin,63)
	get_pcvar_string(amx_rslot_hltv_bantime,bantime,31)
	set_hudmessage(200, 0, 0, -1.0, 0.30, 0, 6.0, 6.0, 0.5, 0.15, -1)
	show_hudmessage(0, "%L", LANG_PLAYER, "WARN_ALL", banin, bantime)
}

public banHLTV(id) {
	if ( g_ban[id] == true ) {
		if (get_pcvar_num(amx_rslot_hltv_bantime) < 2) {
			new lReason[128]
			format(lReason, 127, "%L", id, "BAN_REASON")
			server_cmd("amx_rslot_hltv_bantime 2;amx_banip #%d 2 %s", get_user_userid(id), lReason)
		} else if (get_pcvar_num(amx_rslot_hltv_banin) > 60) {
			new lReason[128]
			format(lReason, 127, "%L", id, "BAN_REASON")
			server_cmd("amx_rslot_hltv_bantime 60;amx_banip #%d 60 %s", get_user_userid(id), lReason)
		} else {
			new lReason[128], bantime[32]
			format(lReason, 127, "%L", id, "BAN_REASON")
			get_pcvar_string(amx_rslot_hltv_bantime,bantime,31)
			server_cmd("amx_banip #%d %s %s", get_user_userid(id), bantime, lReason)
		}
	}
}

public setVisibleSlotsP() {
	set_cvar_num("amx_rslot_hltvimm", 0)
	new visiblemax = get_cvar_num("sv_visiblemaxplayers")
	new players = get_playersnum(1)
	if (!(g_maxplayers = players)) {
		server_cmd("sv_visiblemaxplayers %s", visiblemax+1)
	} else if (!(get_cvar_num("sv_visiblemaxplayers") == -1)) {
		server_cmd("sv_visiblemaxplayers %s", visiblemax+1)
	}
}

public client_authorized(id) {
	new players = get_playersnum(1)
	new limit, who
	new amx_reservation_var = get_cvar_num("amx_reservation")
	if(amx_reservation_var <= 0) return PLUGIN_CONTINUE
	
	if(amx_reservation_var < 4) {
		limit = g_maxplayers - 1
		if(players > limit) {
			if(get_cvar_num("amx_rslot_hltvimm") == 1) {
				if(get_user_flags(id) & ADMIN_RESERVATION) {
					switch(amx_reservation_var) {
						case 1:
						who = kickShortestPlayingTime()
						case 2:
						who = kickLongestPlayingTime()
						case 3:
						who = kickLag()
					}
					if(who) {
						new name[32]
						get_user_name(who, name, 31)
						client_print(0, print_chat, "%L", LANG_PLAYER, "WAS_KICKED", name)
						client_print(id, print_console, "%L", id, "WAS_KICKED", name)
					}
					return PLUGIN_CONTINUE
				} else if(is_user_hltv(id)) {
					switch(amx_reservation_var) {
						case 1:
						who = kickShortestPlayingTime()
						case 2:
						who = kickLongestPlayingTime()
						case 3:
						who = kickLag()
					}
					if(who) {
						new name[32]
						get_user_name(who, name, 31)
						client_print(0, print_chat, "%L", LANG_PLAYER, "WAS_KICKED", name)
						client_print(id, print_console, "%L", id, "WAS_KICKED", name)
					}
					return PLUGIN_CONTINUE
				}
			} else if(get_user_flags(id) & ADMIN_RESERVATION) {
				switch(amx_reservation_var) {
					case 1:
					who = kickShortestPlayingTime()
					case 2:
					who = kickLongestPlayingTime()
					case 3:
					who = kickLag()
				}
				if(who) {
					new name[32]
					get_user_name(who, name, 31)
					client_print(0, print_chat, "%L", LANG_PLAYER, "WAS_KICKED", name)
					client_print(id, print_console, "%L", id, "WAS_KICKED", name)
				}
				return PLUGIN_CONTINUE
			}
			kick(id)
			return PLUGIN_HANDLED
		}
	} else if(amx_reservation_var == 4) {
		limit = g_maxplayers - get_cvar_num("amx_reservedslots")
		if(get_cvar_num("amx_rslot_hltvimm") == 1) {
			if((get_user_flags(id) & ADMIN_RESERVATION) || (players <= limit)) {
				if(get_cvar_num("amx_hideslots") == 1) {
					setVisibleSlots(players, limit)
				}
				return PLUGIN_CONTINUE
			} else if((is_user_hltv(id)) || (players <= limit)) {
				if(get_cvar_num("amx_hideslots") == 1) {
					setVisibleSlots(players, limit)
				}
				return PLUGIN_CONTINUE
			}
		} else if((get_user_flags(id) & ADMIN_RESERVATION) || (players <= limit)) {
			if(get_cvar_num("amx_hideslots") == 1) {
				setVisibleSlots(players, limit)
			}
			return PLUGIN_CONTINUE
		}
		kick(id)
	}
	return PLUGIN_CONTINUE
}

public client_disconnect(id) {
	g_ban[id] = false
	if(get_cvar_num("amx_reservation") == 4 && get_cvar_num("amx_hideslots") == 1) {
		setVisibleSlots(get_playersnum(1) - 1, g_maxplayers - get_cvar_num("amx_reservedslots"))
	}
	return
}

kickShortestPlayingTime() {
	new who = 0, itime, shortest = 0x7fffffff
	for(new i = 1; i <= g_maxplayers; ++i) {
		if(!is_user_connected(i) && !is_user_connecting(i))
		continue
		if(get_user_flags(i) & ADMIN_RESERVATION)
		continue
		if(get_cvar_num("amx_rslot_hltvimm") == 1) {
			if(is_user_hltv(i))
			continue
		}
		itime = get_user_time(i)
		if(shortest > itime) {
			shortest = itime
			who = i
		}
	}
	if(who) {
		kickS(who)
	}
	return who
}

kickLongestPlayingTime() {
	new who = 0, itime, longest = -1
	for(new i = 1; i <= g_maxplayers; ++i) {
		if(!is_user_connected(i) && !is_user_connecting(i))
		continue
		if(get_user_flags(i) & ADMIN_RESERVATION)
		continue
		if(get_cvar_num("amx_rslot_hltvimm") == 1) {
			if(is_user_hltv(i))
			continue
		}
		itime = get_user_time(i)
		if(longest < itime) {
			longest = itime
			who = i
		}
	}
	if(who) {
		kickL(who)
	}
	return who
}

kickLag() {
	new who = 0, ping, loss, worst = -1
	for(new i = 1; i <= g_maxplayers; ++i) {
		if(!is_user_connected(i) && !is_user_connecting(i))
		continue
		if(get_user_flags(i) & ADMIN_RESERVATION)
		continue
		if(get_cvar_num("amx_rslot_hltvimm") == 1) {
			if(is_user_hltv(i))
			continue
		}
		get_user_ping(i, ping, loss)
		if(ping > worst) {
			worst = ping
			who = i
		}
	}
	if(who) {
		kickP(who)
	}
	return who
}

public setVisibleSlots(players, limit) {
	new num = players + 1
	if(players == g_maxplayers) {
		num = g_maxplayers
	} else if(players < limit) {
		num = limit
	}
	set_cvar_num("sv_visiblemaxplayers", num)
}
